$EJECT





			;*****************************
			;    UNIVERSAL LCD DRIVER
			;FOR OPTEX 1 TO 4 LINE DISPLAY
			;PORT 1 DATA , PORT 3.3-5 CONT
			; Randy G. Ferrell  02/27/90
			;    USING ACALL'S & AJMP'S 
			; FOR USE WITH THE 87C751
			;*****************************
			
$EJECT
;*****************************************************************************
	BSEG
READ	BIT	P3.3		;READ=SET & WRITE=RESET
STROBE	BIT	P3.5		;DATA STROBE (ENABLE)
CTRL	BIT	P3.4		;CONTROL BIT (CONTROL=RESET & DATA=SET)
;*****************************************************************************
LCD_DATA	SEGMENT DATA		
	RSEG	LCD_DATA
OFFSET:	DS	1

;*****************************************************************************
lcd_code	segment CODE
	RSEG	LCD_CODE
;*****************************************************************************
PUBLIC	INITLCD, OUTLCD, DELAY, CLEARD   
;***************************CONTROL EQUATES***********************************
INITD	EQU	38H		;INIT. DISPLAY
CUROND	EQU	0EH		;CURSOR ON & DISPLAY ON
CURHOME	EQU	02H		;HOME CURSOR ON DISPLAY
DISPOFF	EQU	08H		;DISPLAY OFF (DATA STAYS INTACT)
DISPON	EQU	0CH		;DISPLAY ON & CURSOR OFF
DISPCLR	EQU	01H		;CLEAR DISPLAY & HOME CURSOR
L1STRT	EQU	00		;LINE 1 CURSOR START
DPORT	EQU	P1		;DISPLAY DATA PORT 1
LCOUNT	EQU	020		;CHARACTERS PER LINE
EOT	EQU	04H		;END OF TEST CODE (HEX)
;***************************************************************************
;
;************************CURSOR MOVEMENT COMMANDS***************************
;
CMLD	EQU	06H		;CURSOR MOVE LEFT WHEN DATA REC.
CMRD	EQU	04H		;CURSOR MOVE RIGHT WHEN DATA REC.
CML	EQU	10H		;CURSOR MOVE LEFT
CMR	EQU	14H		;CURSOR MOVE RIGHT
DML	EQU	18H		;DATA MOVE LEFT
DMR	EQU	1CH		;DATA MOVE RIGHT
;***************************************************************************
;
;*****************************DDRAM EQUATES*********************************
;
DADD	EQU	80H		;DISPLAY CODE ADD ADDERSS (0-40)
;****************************************************************************
$EJECT
;*****************************************************************************
;               
; FUNCTION: INITILIZE LCD     
;               
;   INPUTS: # OF LINES,# OF CHARTERS PER LINE, TYPE DISPLAY
;              
;  OUTPUTS: INIT DISPLAY,CLEAR DISPLAY,HOME CURSOR
;               
;    CALLS: enable,wait40  
;               
; DESTROYS: PORT 1, PORT 3  
;
;*****************************************************************************
;*********************** INITIALIZATION ROUTINE *******************************
INITLCD:	CLR	STROBE		; CLEAR STROBE
	CLR	READ		; CLEAR READ LINE
	CLR	CTRL		; CLEAR CONTROL
	ACALL	WAIT40
	MOV	DPORT,#INITD	; INIT #4  1/16 DUTY  5X7 DOTS
	ACALL	ENABLE
	CLR	CTRL
	ACALL	WAIT40
	MOV	DPORT,#CUROND	;CURSOR ON & DISPLAY ON  
	ACALL	ENABLE
	CLR	CTRL
	ACALL	WAIT40
	MOV	DPORT,#CMLD	;CURSOR MOVE LEFT WHEN DATA IS RECIVED
	ACALL 	ENABLE
	CLR	CTRL
	MOV	DPORT,#DISPCLR		;CLEAR DISPLAY
	ACALL	ENABLE
	ACALL	WAIT40
	SETB	CTRL		;GET READY FOR DATA
	RET
;******************************************************************************
$EJECT
;*****************************************************************************
;               
; FUNCTION: OUTPUTS STRING TO LCD.
;               
;   INPUTS:      DATA POINTER POINTS TO STRING
;    (STRING MUST INCLUDE LINE #0-3 AS FIRST BYTE & END WITH A 4)
;              
;  OUTPUTS:   
;               
;    CALLS: MDIS, SETCUR  
;               
; DESTROYS: NONE   
;
;*****************************************************************************
;
;******************************************************************************
OUTLCD:	PUSH	DPL
	PUSH	DPH
	ACAll	SETCUR		;SETUP CURSOR FOR STRING
	POP	DPH
	POP	DPL
	INC	DPTR
	ACALL	MDISP		;MOVE TO DISPLAY
	RET
;******************************************************************************
$EJECT
;*****************************************************************************
;               
; FUNCTION: CALCUALTE CENTER OF LINE
;               
;   INPUTS: # OF CHARTERS TO DISPLAY  
;               
;  OUTPUTS: OFFSET OF MESSAGE (POSITION CURSOR)
;               
;    CALLS: MOVCUR,HOMECUR,MESSLEN  
;               
; DESTROYS: ACC, B REG, R6, R7
;
;*****************************************************************************
;
SETCUR:	ACALL	MESSLEN		;CALCUATE MESSAGE LENGTH
	MOV	A,#lcount
	SUBB	A,R6		;GET THE DIFFERANCE
	CLR	C
	RRC	A
SKIP:	ADD	A,R7		;GET MESSAGE LINE # (WHAT LINE TO DISPLAY IT ON)
	MOV	OFFSET,A	;SAVE OFFSET
	ACALL	MOVCUR		;PUT CURSOR TO START OF TEXT
	RET
;******************************************************************************
$EJECT
;*****************************************************************************
;               
; FUNCTION: CALCULATE MESSAGE LENGTH & LINE #   
;               
;   INPUTS: DATA POINTER POINTS TO MESSAGE  
;               
;  OUTPUTS: MESSAGE LENGTH IN R6 ,LINE ADDRESS IN R7 
;               
;    CALLS:LINECHK,CHK2,CHK3,CHK4,CHKD,UPDATE   
;               
; DESTROYS: R6, R7, DATA POINTER   
;
;*****************************************************************************
LINECHK:CJNE	A,#00,CHK2	;IS IT LINE 1
	MOV	R7,#00h	;OFFSET FOR LINE #1
	INC	DPTR		;POINT TO FIRST CHAR.
	RET			;EXIT IF LINE #1
CHK2:	CJNE	A,#01,CHK3	;IS IT LINE 2
	MOV	R7,#40h		;OFFSET FOR LINE #2
	INC	DPTR		;POINT TO FIRST CHAR.
	RET			;EXIT IF LINE #2
CHK3:	CJNE	A,#02,CHK4	;IS IT LINE 3
	MOV	R7,#14h		;OFFSET FOR LINE #3
	INC	DPTR		;POINT TO FIRST CHAR.
	RET			;EXIT IF LINE #3
CHK4:	CJNE	A,#03,CHKD	;IS IT LINE 4
	MOV	R7,#54h		;OFFSET FOR LINE #4
	INC	DPTR		;POINT TO FIRST CHAR.
CHKD:	RET			;EXIT IF LINE #4
;***************************************************************************
UPDATE:	INC	R6		;LOAD INDERECT TO ACC
	INC	DPTR
	AJMP	LOOP2
MESSLEN:
	MOV	R6,#00H		;ZERO BYTE COUNT
	MOV	R7,#00H		;DEFALT IS LINE 1
LOOP2:	CLR	A		;CLEAR THE ACC
	MOVC	A,@A+DPTR	;GET CODE BYTE
	ACALL	LINECHK		;GET LINE OFFSET
	CJNE	A,#EOT,UPDATE	;CHECK FOR END OF TEXT
	RET
;****************************************************************************
$EJECT
;*****************************************************************************
;               
; FUNCTION:MOVE THE CURSOR STARTING PLACE   
;               
;   INPUTS:PLACE TO MOVE TO IS POINTED TO BY "OFFSET"   
;               
;  OUTPUTS:   
;               
;    CALLS:ENABLE,WAIT40   
;               
; DESTROYS:   
;
;*****************************************************************************
MOVCUR:	CLR	CTRL		;SETUP FOR CONTROL CODES
	MOV	A,OFFSET
	ADD	a,#dadd		;add control code
	MOV	DPORT,A		;ADJUST FOR OFFSET
	ACALL	ENABLE		;STROBE DISPLAY
	ACALL	WAIT40		;GIVE TIME
	SETB	CTRL
	RET
;
$EJECT
;*****************************************************************************
;               
; FUNCTION: CLEAR LCD AND HOME CURSOR
;               
;   INPUTS: NONE  
;               
;  OUTPUTS:   
;               
;    CALLS: ENABLE,WAIT40  
;               
; DESTROYS:   
;
;*****************************************************************************
;
CLEARD:	CLR	CTRL
	MOV	DPORT,#DISPCLR
	ACALL	ENABLE
	ACALL	WAIT40
	SETB	CTRL
	RET
;******************************************************************************
$EJECT
;*****************************************************************************
;               
; FUNCTION: MOVE MESSAGE TO LCD AT CURRENT CURSOR  
;               
;   INPUTS:MESLEN CONTAINS LENGTH, DATA POINTER POINTS TO MESSAGE.
;               
;  OUTPUTS:   
;               
;    CALLS:ENABLE   
;               
; DESTROYS: ACC,DPTR   
;
;*****************************************************************************
;
;
MDISP:	CLR	A
	MOVC	A,@A+DPTR	;GET CHAR.
	MOV	DPORT,A		;PUT BYTE ON PORT
	ACALL	ENABLE		;FIRE STROBE
	INC	DPTR		;ADD ONE TO POINTER
	DJNZ	R6,MDISP		;LOOP UNTIL ALL DATA OUT
	RET
;******************************************************************************
$EJECT
;*****************************************************************************
;               
; FUNCTION: SENDS DATA TO LCD AS FAST AS IT CAN TAKE IT  
;               
;   INPUTS:   
;               
;  OUTPUTS:   
;               
;    CALLS:   
;               
; DESTROYS:   
;
;*****************************************************************************
;
ENABLE:	SETB	STROBE
	NOP
 	NOP
	CLR	STROBE
	CLR	CTRL
	MOV	DPORT,#255	;SET UP BIT FOR READ
	SETB	READ		;LOOK FOR BUSY FLAG
WAITL:	SETB	STROBE
	NOP
	JB	P1.7,WAITL
	CLR	STROBE
	CLR	READ
	SETB	CTRL	
	RET
;******************************************************************************
$EJECT
;*****************************************************************************
;               
; FUNCTION: DELAY FOR FLASH & SCROLL OR JUST DELAY
;               
;   INPUTS: R4 CONTROLS AMOUNT OF DELAY  
;               
;  OUTPUTS:   
;               
;    CALLS:   
;               
; DESTROYS: R2, R3, R4,   
;
;*****************************************************************************
;
;******************************************************************************
WAIT40:	MOV	R3,#35		; 30MS DELAY LOOP
WAIT1:	MOV	R2,#255		; 1.4MS INSIDE DELAY LOOP
WAIT2:	DJNZ	R2,WAIT2
	DJNZ	R3,WAIT1
	RET
;*****************************************************************************
DELAY:	ACALL	WAIT40
	DJNZ	R4,DELAY
	RET
;******************************************************************************
	END